"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const index_js_1 = require("./index.js");
const luxon_1 = require("luxon");
luxon_1.Settings.defaultZone = "UTC";
const vitest_1 = require("vitest");
const dateTimeRef = luxon_1.DateTime.utc(2022, 1, 1);
function log(value) {
    console.dir(value, { depth: null });
}
(0, vitest_1.test)("Basic", () => __awaiter(void 0, void 0, void 0, function* () {
    // true, true
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: true,
        },
    ]);
}));
(0, vitest_1.test)("Basic available", () => __awaiter(void 0, void 0, void 0, function* () {
    // false, true
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        availability: [
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: true,
        },
    ]);
}));
(0, vitest_1.test)("Basic unavailable", () => __awaiter(void 0, void 0, void 0, function* () {
    // true, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        unavailability: [
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: false,
        },
    ]);
}));
(0, vitest_1.test)("Basic available and unavailable", () => __awaiter(void 0, void 0, void 0, function* () {
    // false, true, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 3 }).toISO(),
        availability: [
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
            },
        ],
        unavailability: [
            {
                from: dateTimeRef.plus({ hour: 2 }).toISO(),
                to: dateTimeRef.plus({ hour: 3 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T02:00:00.000Z",
            to: "2022-01-01T03:00:00.000Z",
            available: false,
        },
    ]);
}));
(0, vitest_1.test)("Available and unavailable without from and to", () => __awaiter(void 0, void 0, void 0, function* () {
    // true, false
    const { allSlots } = (0, index_js_1.getSlots)({
        availability: [
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
            },
        ],
        unavailability: [
            {
                from: dateTimeRef.plus({ hour: 2 }).toISO(),
                to: dateTimeRef.plus({ hour: 3 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T02:00:00.000Z",
            to: "2022-01-01T03:00:00.000Z",
            available: false,
        },
    ]);
}));
(0, vitest_1.test)("Some time between unavailable and available slot", () => __awaiter(void 0, void 0, void 0, function* () {
    // false, true, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 3, minute: 30 }).toISO(),
        availability: [
            {
                from: dateTimeRef.plus({ hour: 1, minute: 30 }).toISO(),
                to: dateTimeRef.plus({ hour: 2, minute: 30 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T01:30:00.000Z",
            to: "2022-01-01T02:30:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T02:30:00.000Z",
            to: "2022-01-01T03:30:00.000Z",
            available: false,
        },
    ]);
}));
(0, vitest_1.test)("Show unavailable slot when there if there are available minutes but it won't lead to a full slot", () => __awaiter(void 0, void 0, void 0, function* () {
    // false, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2, minute: 30 }).toISO(),
        availability: [
            {
                from: dateTimeRef.plus({ hour: 1, minute: 30 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: false,
        },
    ]);
}));
(0, vitest_1.test)("User availabilities", () => {
    // true + Alice, true + Bob
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        availability: [
            {
                day: "Saturday",
                from: "00:00",
                to: "01:00",
                metadata: {
                    user: "Alice",
                },
            },
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
                metadata: {
                    user: "Bob",
                },
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
            metadataAvailable: [{ user: "Alice" }],
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: true,
            metadataAvailable: [{ user: "Bob" }],
        },
    ]);
});
(0, vitest_1.test)("User availabilities and unavailabilities", () => {
    // true + Alice, false + Alice
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        availability: [
            {
                from: dateTimeRef.toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
                metadata: {
                    user: "Alice",
                },
            },
        ],
        unavailability: [
            {
                from: dateTimeRef.plus({ hour: 1 }).toISO(),
                to: dateTimeRef.plus({ hour: 2 }).toISO(),
                metadata: {
                    user: "Alice",
                },
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
            metadataAvailable: [{ user: "Alice" }],
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: false,
            metadataUnavailable: [{ user: "Alice" }],
        },
    ]);
});
(0, vitest_1.test)("With timezones", () => {
    // true + offset 1, false + offset 1
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        outputTimezone: "Europe/Paris",
        availability: [
            {
                day: "Saturday",
                from: "01:00",
                to: "02:00",
                timezone: "Europe/Paris",
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T01:00:00.000+01:00",
            to: "2022-01-01T02:00:00.000+01:00",
            available: true,
        },
        {
            from: "2022-01-01T02:00:00.000+01:00",
            to: "2022-01-01T03:00:00.000+01:00",
            available: false,
        },
    ]);
});
(0, vitest_1.test)("Day slot does not fall within bounds", () => {
    // false, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        availability: [
            {
                day: "Saturday",
                from: "16:00",
                to: "17:00",
                timezone: "America/Los_Angeles",
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: false,
        },
    ]);
});
(0, vitest_1.test)("Bounds and day slot aren't the same timezone, so days of the week differ", () => {
    // true, false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.toISO(),
        to: dateTimeRef.plus({ hour: 2 }).toISO(),
        availability: [
            {
                day: "Friday",
                from: "16:00",
                to: "17:00",
                timezone: "America/Los_Angeles",
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T01:00:00.000Z",
            to: "2022-01-01T02:00:00.000Z",
            available: false,
        },
    ]);
});
(0, vitest_1.test)("Day slot across Daylight savings time", () => {
    const { allSlots } = (0, index_js_1.getSlots)({
        from: luxon_1.DateTime.fromObject({ year: 2024, month: 3, day: 10, hour: 1 }, { zone: "America/Los_Angeles" }).toISO(),
        to: luxon_1.DateTime.fromObject({ year: 2024, month: 3, day: 10, hour: 7 }, { zone: "America/Los_Angeles" }).toISO(),
        availability: [
            {
                day: "Sunday",
                from: "05:00",
                to: "06:00",
                timezone: "America/Los_Angeles",
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        // PST is -8
        {
            from: luxon_1.DateTime.fromISO("2024-03-10T01:00:00.000-08:00").toUTC().toISO(),
            to: luxon_1.DateTime.fromISO("2024-03-10T02:00:00.000-08:00").toUTC().toISO(),
            available: false,
        },
        // The 2-3 slot doesn't exist
        // PDT is -7
        {
            from: luxon_1.DateTime.fromISO("2024-03-10T03:00:00.000-07:00").toUTC().toISO(),
            to: luxon_1.DateTime.fromISO("2024-03-10T04:00:00.000-07:00").toUTC().toISO(),
            available: false,
        },
        {
            from: luxon_1.DateTime.fromISO("2024-03-10T04:00:00.000-07:00").toUTC().toISO(),
            to: luxon_1.DateTime.fromISO("2024-03-10T05:00:00.000-07:00").toUTC().toISO(),
            available: false,
        },
        {
            from: luxon_1.DateTime.fromISO("2024-03-10T05:00:00.000-07:00").toUTC().toISO(),
            to: luxon_1.DateTime.fromISO("2024-03-10T06:00:00.000-07:00").toUTC().toISO(),
            available: true,
        },
        {
            from: luxon_1.DateTime.fromISO("2024-03-10T06:00:00.000-07:00").toUTC().toISO(),
            to: luxon_1.DateTime.fromISO("2024-03-10T07:00:00.000-07:00").toUTC().toISO(),
            available: false,
        },
    ]);
});
(0, vitest_1.test)("Helper variables", () => __awaiter(void 0, void 0, void 0, function* () {
    // false, true
    const { allSlots, availableSlots, allDates, availableDates, allSlotsByDay, availableSlotsByDay, } = (0, index_js_1.getSlots)({
        from: dateTimeRef.minus({ hour: 1 }).toISO(),
        to: dateTimeRef.plus({ hour: 1 }).toISO(),
        availability: [
            {
                from: dateTimeRef.toISO(),
                to: dateTimeRef.plus({ hour: 1 }).toISO(),
            },
        ],
        duration: 60,
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2021-12-31T23:00:00.000Z",
            to: "2022-01-01T00:00:00.000Z",
            available: false,
        },
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
        },
    ]);
    (0, vitest_1.expect)(availableSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000Z",
            to: "2022-01-01T01:00:00.000Z",
            available: true,
        },
    ]);
    (0, vitest_1.expect)(allDates).toEqual(["2021-12-31", "2022-01-01"]);
    (0, vitest_1.expect)(availableDates).toEqual(["2022-01-01"]);
    (0, vitest_1.expect)(allSlotsByDay).toEqual({
        "2021-12-31": [
            {
                from: "2021-12-31T23:00:00.000Z",
                to: "2022-01-01T00:00:00.000Z",
                available: false,
            },
        ],
        "2022-01-01": [
            {
                from: "2022-01-01T00:00:00.000Z",
                to: "2022-01-01T01:00:00.000Z",
                available: true,
            },
        ],
    });
    (0, vitest_1.expect)(availableSlotsByDay).toEqual({
        "2022-01-01": [
            {
                from: "2022-01-01T00:00:00.000Z",
                to: "2022-01-01T01:00:00.000Z",
                available: true,
            },
        ],
    });
}));
(0, vitest_1.test)("Weekdays are in a specified locale", () => {
    // true false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.plus({ hour: 3 }).toISO(),
        to: dateTimeRef.plus({ hour: 5 }).toISO(),
        outputTimezone: "America/Sao_Paulo",
        duration: 60,
        availability: [
            {
                day: {
                    text: "sábado",
                    locale: "pt-br",
                },
                from: "00:00",
                to: "01:00",
                timezone: "America/Sao_Paulo",
            },
        ],
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T00:00:00.000-03:00",
            to: "2022-01-01T01:00:00.000-03:00",
            available: true,
        },
        {
            from: "2022-01-01T01:00:00.000-03:00",
            to: "2022-01-01T02:00:00.000-03:00",
            available: false,
        },
    ]);
});
(0, vitest_1.test)("Availability bounds include from and to", () => {
    // true false
    const { allSlots } = (0, index_js_1.getSlots)({
        from: dateTimeRef.plus({ hour: 8 }).toISO(),
        to: dateTimeRef.plus({ hour: 10 }).toISO(),
        duration: 60,
        availability: [
            {
                from: dateTimeRef.toISO(),
                to: dateTimeRef.plus({ hour: 23 }).toISO(),
            },
        ],
    });
    (0, vitest_1.expect)(allSlots).toEqual([
        {
            from: "2022-01-01T08:00:00.000Z",
            to: "2022-01-01T09:00:00.000Z",
            available: true,
        },
        {
            from: "2022-01-01T09:00:00.000Z",
            to: "2022-01-01T10:00:00.000Z",
            available: true,
        },
    ]);
});
